#!/usr/bin/perl

use Math::Complex;

$S11_dev = cplx(1, 1);
$S12_dev = cplx(1, 2);
$S21_dev = cplx(2, 1);
$S22_dev = cplx(2, 2);
print "Init S = $S11_dev\, $S12_dev\, $S21_dev\, $S22_dev\n";

($Y11_dev, $Y12_dev, $Y21_dev, $Y22_dev) = StoY($S11_dev, $S12_dev, $S21_dev, $S22_dev);
print "S -> Y = $Y11_dev\, $Y12_dev\, $Y21_dev\, $Y22_dev\n";

($S11_dev, $S12_dev, $S21_dev, $S22_dev) = YtoS($Y11_dev, $Y12_dev, $Y21_dev, $Y22_dev);
print "Y -> S = $S11_dev\, $S12_dev\, $S21_dev\, $S22_dev\n";

($Z11_dev, $Z12_dev, $Z21_dev, $Z22_dev) = StoZ($S11_dev, $S12_dev, $S21_dev, $S22_dev);
print "S -> Z = $Z11_dev\, $Z12_dev\, $Z21_dev\, $Z22_dev\n";

($S11_dev, $S12_dev, $S21_dev, $S22_dev) = ZtoS($Z11_dev, $Z12_dev, $Z21_dev, $Z22_dev);
print "Z -> S = $S11_dev\, $S12_dev\, $S21_dev\, $S22_dev\n";

($Z11_dev, $Z12_dev, $Z21_dev, $Z22_dev) = YtoZ($Y11_dev, $Y12_dev, $Y21_dev, $Y22_dev);
print "Y -> Z = $Z11_dev\, $Z12_dev\, $Z21_dev\, $Z22_dev\n";

($Y11_dev, $Y12_dev, $Y21_dev, $Y22_dev) = ZtoY($Z11_dev, $Z12_dev, $Z21_dev, $Z22_dev);
print "Z -> Y = $Y11_dev\, $Y12_dev\, $Y21_dev\, $Y22_dev\n";

# Convert S to Y
sub StoY {
  ($s11, $s12, $s21, $s22) = @_;
  $y0 = 1 / 50;
  $ss = (1 + $s11) * (1 + $s22) - $s12 * $s21;
  $y11 = $y0 * ((1 - $s11) * (1 + $s22) + $s12 * $s21) / $ss;
  $y12 = - $y0 * 2 * $s12 / $ss;
  $y21 = - $y0 * 2 * $s21 / $ss;
  $y22 = $y0 * ((1 + $s11) * (1 - $s22) + $s12 * $s21) / $ss;
  return($y11, $y12, $y21, $y22);
}
# Convert Y to S
sub YtoS {
  ($y11, $y12, $y21, $y22) = @_;
  $y0 = 1 / 50;
  $yy = (1 + $y11/$y0) * (1 + $y22/$y0) - ($y12/$y0) * ($y21/$y0);
  $s11 = ((1 - $y11/$y0) * (1 + $y22/$y0) + ($y12/$y0) * ($y21/$y0)) / $yy;
  $s12 = -2 * ($y12/$y0) / $yy;
  $s21 = -2 * ($y21/$y0) / $yy;
  $s22 = ((1 + $y11/$y0) * (1 - $y22/$y0) + ($y12/$y0) * ($y21/$y0)) / $yy;
  return($s11, $s12, $s21, $s22);
}
# Convert S to Z
sub StoZ {
  ($s11, $s12, $s21, $s22) = @_;
  $z0 = 50;
  $ss = (1 - $s11) * (1 - $s22) - $s12 * $s21;
  $z11 = $z0 * ((1 + $s11) * (1 - $s22) + $s12 * $s21) / $ss;
  $z12 = $z0 * 2 * $s12 / $ss;
  $z21 = $z0 * 2 * $s21 / $ss;
  $z22 = $z0 * ((1 - $s11) * (1 + $s22) + $s12 * $s21) / $ss;
  return($z11, $z12, $z21, $z22);
}
# Convert Z to S
sub ZtoS {
  ($z11, $z12, $z21, $z22) = @_;
  $z0 = 50;
  $zz = ($z11/$z0 + 1) * ($z22/$z0 + 1) - ($z12/$z0) * ($z21/$z0);
  $s11 = (($z11/$z0 - 1) * ($z22/$z0 + 1) - ($z12/$z0) * ($z21/$z0)) / $zz;
  $s12 = 2 * ($z12/$z0) / $zz;
  $s21 = 2 * ($z21/$z0) / $zz;
  $s22 = (($z11/$z0 + 1) * ($z22/$z0 - 1) - ($z12/$z0) * ($z21/$z0)) / $zz;
  return($s11, $s12, $s21, $s22);
}
# Convert Y to Z
sub YtoZ {
  ($y11, $y12, $y21, $y22) = @_;
  $yy = $y11 * $y22 - $y12 * $y21;
  $z11 = $y22 / $yy;
  $z12 = - $y12 / $yy;
  $z21 = - $y21 / $yy;
  $z22 = $y11 / $yy;
  return($z11, $z12, $z21, $z22);
}
# Convert Z to Y
sub ZtoY {
  ($z11, $z12, $z21, $z22) = @_;
  $zz = $z11 * $z22 - $z12 * $z21;
  $y11 = $z22 / $zz;
  $y12 = - $z12 / $zz;
  $y21 = - $z21 / $zz;
  $y22 = $z11 / $zz;
  return($y11, $y12, $y21, $y22);
}
